<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Helper;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class AuthController extends Controller
{
    public function adminLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|string',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $credentials = $request->only('email', 'password');

        if (!Auth::guard('admin')->attempt($credentials)) {
            return Helper::return(['message' => __('general.unauthorized')], Response::HTTP_UNAUTHORIZED);
        }
        $admin = Auth::guard('admin')->user();
        $token  =  $admin->createToken('auth')->plainTextToken;
        return Helper::return(['message' => __('general.welcome'), 'data' => ['user_info' => $admin, 'token' => $token]], Response::HTTP_OK);
    }
    public function changePassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'current_password' => 'required|string',
            'password' => 'required|min:8|confirmed',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        return Hash::check($request->current_password, $admin->password);
        if (!Hash::check($request->current_password, $admin->password))
            return Helper::return(['message' => __('general.current_password_wrong')], Response::HTTP_UNAUTHORIZED);

        $admin->update(['password' => Hash::make($request->password)]);
        return Helper::return(['message' => __('general.successfuly_updated', ['entity' => 'Password'])], Response::HTTP_OK);
    }
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string',
            'email' => 'required|email|unique:users',
            'password' => 'required|string',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $input = $validator->validated();
        $input['password'] = Hash::make($input['password']);
        $user = User::create($validator->validated());
        $token  =  $user->createToken('auth')->plainTextToken;
        return Helper::return(['message' => __('general.welcome'), 'data' => ['user_info' => $user, 'token' => $token]], Response::HTTP_OK);
    }
    public function userLogin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|string',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $credentials = $request->only('email', 'password');

        if (!Auth::guard('user')->attempt($credentials)) {
            return Helper::return(['message' => __('general.unauthorized')], Response::HTTP_UNAUTHORIZED);
        }
        $user = Auth::guard('user')->user();

        $token  =  $user->createToken('auth')->plainTextToken;
        return Helper::return(['message' => __('general.welcome'), 'data' => ['user_info' => $user, 'token' => $token]], Response::HTTP_OK);
    }
}
