<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Helper;
use App\Models\Category;
use App\Models\CategoryDetail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class CategoryController extends Controller
{
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string',
            'type' => 'required|string|in:Tags,Stack,Type Face',
            'title' => 'string|required_if:type,Tags',
            'page_title' => 'string|required_if:type,Tags',
            'page_description' => 'string|required_if:type,Tags',
            'meta_description' => 'string|required_if:type,Tags',
            'icon' => 'file',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        if (Category::where(['name' => $request->name, 'type' => $request->type])->exists())
            return Helper::return(['message' => __('general.duplicate_entry', ['entity' => 'Category'])], Response::HTTP_BAD_REQUEST);
        $index = Category::where('type', $request->type)->max('index');
        $input = $request->only('name', 'type');
        $index == null ? $index = 1 : $index = $index + 1;
        $input['index'] = $index;
        $category = Category::create($input);
        if ($request->has('icon')) {
            Helper::storeFile([['file' => $request->file('icon'), 'info' => ['type' => 'Category', 'section' => 'icon', 'sub_section' => '', 'item_id' => $category->id, 'index' => null]]]);
        }
        if ($request->input('type') == 'Tags') {
            $input = [];
            $input = ['title' => $request->title, 'page_title' => $request->page_title, 'page_description' => $request->page_description, 'meta_description' => $request->meta_description];
            $input['category_id'] = $category->id;
            CategoryDetail::create($input);
        }
        return Helper::return(['message' => __('general.created', ['entity' => 'Category']), 'data' => Category::with('detail')->find($category->id)], Response::HTTP_OK);
    }
    public function reorder(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'index' => 'required|numeric',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        if (!$category = Category::find($id))
            return Helper::return(['message' => __('general.invalid_id', ['entity' => 'Category']), 'data' => []], Response::HTTP_BAD_REQUEST);
        if (Category::where(['index' => $request->index, 'type' => $category->type])->exists())
            Category::where('index', '>=', $request->index)->where('type', $category->type)->increment('index');
        $category->update(['index' => $request->index]);
        return Helper::return(['message' => __('general.successfuly_updated', ['entity' => 'Category'])], Response::HTTP_OK);
    }
    public function destroy($id)
    {
        if (!$category = Category::find($id))
            return Helper::return(['message' => __('general.invalid_id', ['entity' => 'Category']), 'data' => []], Response::HTTP_BAD_REQUEST);
        $category->delete();
        return Helper::return(['message' => __('general.deleted', ['entity' => 'Category']), 'data' => []], Response::HTTP_OK);
    }
    public function index()
    {
        return Helper::return(['message' => __('general.deleted', ['entity' => 'Category']), 'data' => Category::with('detail', 'icon')->get()], Response::HTTP_OK);
    }
}
