<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Helper;
use App\Models\Invoice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use stdClass;
use Symfony\Component\HttpFoundation\Response;

class PaymentController extends Controller
{
    static public function createPaymentSession(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type' => "required|in:monthly,yearly"
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        if ($request->type == 'monthly') {
            $product = env('STRIPE_MONTHLY_PRO_KEY');
        } elseif ($request->type == 'yearly') {
            $product = env('STRIPE_YEARLY_PRO_KEY');
        }
        $currency = env('CURRENCY');
        $obj = new stdClass();
        try {
            $key = env('STRIPE_KEY');
            $stripe = new \Stripe\StripeClient($key);

            $response = $stripe->checkout->sessions->create([
                'success_url' => 'https://foodline.at/checkoutsuccessful',
                'cancel_url' => 'https://foodline.at/checkoutfailed',
                'mode' => 'payment',
                'line_items' => [
                    [
                        'quantity' => 1,
                        'price_data' => ['currency' => $currency, 'product' => $product,]
                    ],
                ]
            ]);
            $invoice = Invoice::create([
                'user_id' => $request->user()->id,
                'tracking_code' => $response->id,
                'payment_status' => 'pending',
            ]);
            $obj->status = true;
            $obj->status_code = 200;
            $obj->url = $response->url;
            $obj->message = 'URL created';
            return $obj;
        } catch (Exception $e) {
            $obj->status = false;
            $obj->status_code = $e->getHttpStatus();
            $obj->url = null;
            $obj->message = $e->getError()->message;
            $obj->invoice_id = null;
            $obj->discount_id = null;
            return $obj;
        }
    }
}
