<?php

namespace App\Http\Controllers;

use App\Http\Helper;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class UserController extends Controller
{
    public function adminGetList(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'paginate' => 'required|numeric',
            'search' => 'string',
            'sort' => 'string|in:created_at,a-z',
            'status' => 'in:premium',
        ]);
        if ($validator->fails()) {
            return Helper::return(['message' => __('general.validation_failed'), 'data' => $validator->errors()], Response::HTTP_BAD_REQUEST);
        }
        $model = new User;
        if ($request->has('search'))
            $model = $model->where('name', "LIKE", "%$request->search%");
        if ($request->has('status'))
            if ($request->input('status') == 'premium')
                $model = $model->where('is_premium', 1);
        if ($request->has('sort')) {
            if ($request->input('status') == 'created_at') {
                $model = $model->orderBy('ASC', 'created_at');
            } elseif ($request->input('status') == 'a-z') {
                $model = $model->orderBy('ASC', 'name');
            }
        }
        $users = $model->withTrashed()->paginate($request->paginate);
        return Helper::return(['message' => __('general.fetched', ['entity' => 'Users']), 'data' => $users], Response::HTTP_OK);
    }
    public function action($id)
    {
        if (!$user = User::withTrashed()->find($id))
            return Helper::return(['message' => __('general.invalid_id', ['entity' => 'User']), 'data' => []], Response::HTTP_BAD_REQUEST);

        if ($user->deleted_at == null) {
            $user->delete();
        } else {
            $user->deleted_at = null;
            $user->save();
        }
        return Helper::return(['message' => __('general.update', ['entity' => 'Users']), 'data' => $user], Response::HTTP_OK);
    }
    public function detail($id)
    {
        if (!$user = User::withTrashed()->with('invoice')->find($id))
            return Helper::return(['message' => __('general.invalid_id', ['entity' => 'User']), 'data' => []], Response::HTTP_BAD_REQUEST);
        return Helper::return(['message' => __('general.fetched', ['entity' => 'Users']), 'data' => $user], Response::HTTP_OK);
    }
}
