<?php

namespace App\Modules\CRUD\Controller;

use App\Http\Controllers\Controller;
use App\Http\Helper;
use App\Models\File;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\File as FacadesFile;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Schema;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Str;
use stdClass;

class CRUDController extends Controller
{
    static public function createEntities($input, $file_keys, $entity)
    {
        $file = [];
        if (key_exists('response', $input)) {
            $response_type = $input['response'];
            unset($input['response']);
        } else {
            $response_type = '';
        }
        if ($file_keys) {
            $array = Helper::separateFileFromData($input, $file_keys);
            $input = $array['input'];
            $file = $array['file'];
        }
        $created_record = "App\\Models\\$entity"::create($input);
        if ($file) {
            $array = Helper::makeFileStorable($file, $entity, null, $created_record->id);
            self::storeFile($array);
        }
        if ($response_type == 'data')
            return Helper::return(['message' => __('general.created', ['entity' => $entity]), 'data' => $created_record], Response::HTTP_CREATED);

        return Helper::return(['message' => __('general.created', ['entity' => $entity]), 'data' => $created_record->id], Response::HTTP_CREATED);
    }
    static public  function storeFile($file)
    {
        foreach ($file as $value) {
            $id = rand(1, 10000);
            $time = time();
            $name = md5($id . $time . $value['file']->getClientOriginalName());
            $created_file[] = File::create([
                'type' => $value['info']['type'],
                'section' => $value['info']['section'],
                'sub_section' => $value['info']['sub_section'],
                'item_id' => $value['info']['item_id'],
                'address' => $name,
                'original_name' => $value['file']->getClientOriginalName(),
                'extension' => $value['file']->extension(),
                'mime_type' => $value['file']->getClientMimeType(),
            ]);
            $value['file']->storeAs('public', $name . '.' . $value['file']->extension());
        }
        return true;
    }
    static public function indexEntities($inputs,  $entity, $paginate = 'all')
    {
        $inputs = self::formInputs($inputs);
        $model = self::queryBuilder($entity, $inputs);
        $paginate == 'all' ? $data = $model->get() : $data = $model->paginate($paginate);
        return Helper::return(['message' => __('general.fetched', ['entity' => $entity]), 'data' => $data], Response::HTTP_OK);
    }
    static public function showEntities($inputs, $entity, $id)
    {
        $inputs = self::formInputs($inputs);
        $model = self::queryBuilder($entity, $inputs);
        $data = $model->find($id);
        return Helper::return(['message' => __('general.fetched', ['entity' => $entity]), 'data' => $data], Response::HTTP_OK);
    }
    static public function updateEntities($input, $file_keys, $entity, $id)
    {
        $query_requirements = $file = [];
        $input = self::FileNeedsTobeDeleted($input, $entity, $id);
        if ($file_keys) {
            $array = Helper::separateFileFromData($input, $file_keys);
            $input = $array['input'];
            $file = $array['file'];
        }
        unset($input['_method']);
        if (key_exists('with', $input)) {

            $query_requirements['with'] = $input['with'];
            unset($input['with']);
        }
        if (key_exists('withCount', $input)) {
            $query_requirements['withCount'] = $input['withCount'];
            unset($input['withCount']);
        }
        "App\\Models\\$entity"::where('id', $id)->update($input);

        if ($file) {
            $array = Helper::makeFileStorable($file, $entity, null, $id);
            self::updateFile($array, $entity, $id);
        }
        return self::showEntities($query_requirements, $entity, $id);
    }
    static public function FileNeedsTobeDeleted($input, $entity, $id)
    {
        $need_to_delete = $file_section = [];
        $keys = array_keys($input);
        foreach ($keys as $key) {
            if (Str::endsWith($key, '_deleted') && $input[$key]) {
                $need_to_delete[] = $key;
                $file_section[] = Str::before($key, '_deleted');
            }
        }
        $input = Arr::except($input, $need_to_delete);
        if (count($file_section) < 0)
            return $input;
        foreach ($file_section as $section) {
            File::where(['item_id' => $id, 'type' => $entity, 'section' => $section])->delete();
        }
        return $input;
    }
    static public function updateFile($array, $entity, $id)
    {
        foreach ($array as $key => $value) {
            if ($file = File::where('item_id', $id)->where(['type' => $entity, 'section' => $value['info']['section'], 'sub_section' => $value['info']['sub_section']])->first()) {
                FacadesFile::delete(storage_path('app/public/' . $file->address));
                $file->update(Self::storeFileToStorage($value['file']));
            } else {
                self::storeFile([$value]);
            }
        }
        return true;
    }
    static public function storeFileToStorage($file)
    {
        $id = rand(1, 10000);
        $time = time();
        $name = md5($id . $time . $file->getClientOriginalName());
        $file->storeAs('public', $name . '.' . $file->extension());
        return $array = ['address' => $name, 'original_name' => $file->getClientOriginalName(), 'extension' => $file->extension(), 'mime_type' => $file->getClientMimeType()];
    }
    static public function deleteEntities($entity, $delete_type, $id)
    {
        $model = "App\\Models\\$entity";
        if ($delete_type == 'void') {
            $model::where('id', $id)->update(['delete_status' => 'void']);
            File::where('item_id', $id)->where('type', $entity)->delete();
        } else {
            $columns = Schema::getColumnListing((new $model)->getTable());
            if (in_array('delete_status', $columns))
                $model::where('id', $id)->update(['delete_status' => 'deleted']);
            $model::where('id', $id)->delete();
            File::where('id', $id)->where('type', $entity)->delete();
        }
        return Helper::return(['message' => __('general.deleted', ['entity' => $entity]), 'data' => []], Response::HTTP_OK);
    }
    static  public function getEntities($inputs, $entity, $how_to_get)
    {
        $inputs = self::formInputs($inputs);
        $model = self::queryBuilder($entity, $inputs);
        $data = $model->$how_to_get();
        return Helper::return(['message' => __('general.fetched', ['entity' => $entity]), 'data' => $data], Response::HTTP_OK);
    }

    static public function formInputs($input)
    {

        $withRelations =  key_exists('with', $input) ? $input['with'] : [];
        $withCountRelations =  key_exists('withCount', $input) ? $input['withCount'] : [];
        if (key_exists('where', $input)) {
            foreach ($input['where'] as $key => $value) {
                $where[$key] = $value;
            }
        } else {
            $where = [];
        }
        if (key_exists('orWhere', $input)) {
            foreach ($input['orWhere'] as $key => $value) {
                $orWhere[$key] = $value;
            }
        } else {
            $orWhere = [];
        }

        if (key_exists('whereIn', $input)) {
            foreach ($input['whereIn'] as $key => $value) {
                $whereIn[$key] = $value;
            }
        } else {
            $whereIn = [];
        }
        $obj = new stdClass();
        $obj->with = $withRelations;
        $obj->withCount = $withCountRelations;
        $obj->where = $where;
        $obj->orWhere = $orWhere;
        $obj->whereIn = $whereIn;
        return $obj;
    }
    static public function queryBuilder($entity, $inputs)
    {
        $entityClass = "App\\Models\\$entity";
        $model =  new $entityClass;

        if (!empty($inputs->with))
            $model = $model->with($inputs->with);

        if (!empty($inputs->withCount))
            $model = $model->withCount($inputs->withCount);

        if ($inputs->where)
            $model = $model->where($inputs->where);

        if ($inputs->orWhere)
            $model = $model->where($inputs->orWhere);
        if ($inputs->whereIn)
            $model = $model->where(function ($q) use ($inputs) {
                foreach ($inputs->whereIn as $key => $value) {

                    $q->whereIn($key, $value);
                }
            });
        return $model;
    }
}
