<?php

use App\Http\Controllers\API\AuthController;
use App\Http\Controllers\API\BookmarkController;
use App\Http\Controllers\API\CategoryController;
use App\Http\Controllers\API\PaymentController;
use App\Http\Controllers\API\PostController;
use App\Http\Controllers\API\SettingController;
use App\Http\Controllers\UserController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::post('admin/login', [AuthController::class, 'adminLogin']);
Route::post('user/register', [AuthController::class, 'register']);
Route::post('user/login', [AuthController::class, 'userLogin']);
Route::middleware('auth:sanctum')->group(function () {
    Route::middleware('check_admin')->group(function () {

        Route::post('admin/change_password', [AuthController::class, 'changePassword']);
        Route::post('admin/setting', [SettingController::class, 'store']);
        Route::get('admin/setting', [SettingController::class, 'index']);
        Route::post('admin/post', [PostController::class, 'store']);
        Route::get('admin/post/{id}', [PostController::class, 'show']);
        Route::get('admin/browse/post', [PostController::class, 'showAll']);
        Route::get('admin/post/action/{id}', [PostController::class, 'takeAction']);
        Route::post('admin/category', [CategoryController::class, 'store']);
        Route::delete('admin/category/{id}', [CategoryController::class, 'destroy']);
        Route::put('admin/category/reorder/{id}', [CategoryController::class, 'reorder']);
        Route::get('admin/category/index', [CategoryController::class, 'index']);
        Route::get('admin/user/index', [UserController::class, 'adminGetList']);
        Route::get('admin/user/action/{id}', [UserController::class, 'action']);
    });
    Route::middleware('check_user')->group(function () {
        Route::post('bookmark/action/{post_id}', [BookmarkController::class, 'bookmarkAction']);
        Route::get('bookmark/list', [BookmarkController::class, 'list']);
        Route::get('payment/url', [PaymentController::class, 'createPaymentSession']);
    });
});

//Guest OR User
Route::middleware('check_auth')->group(function () {
    Route::get('user/index', [PostController::class, 'index']);
    Route::get('user/post/{id}/show', [PostController::class, 'show']);
});
Route::get('user/category/index', [CategoryController::class, 'index']);
